
/*
   smtp_socket.h
   this is a simple interface to sending E-Mails from any C++ application.
   it only supports the (insecure) LOGIN authentification method so far.
   
   this sourcecode is provided under the terms of the GNU general public
   license version 3. See gpl3.txt for details.
   
   example of usage:
   =================
   SMTP s("server:port", "my_username", "my_password");
   EMail m("my_email_address", "recipients_email_address", "the subject");
   m += "hi there";
   m += "how are you?";
   s.Send(m);
   
   hardcore example:
   =================
   SMTP("server:port", "username", "password").Send(EMail("sender", "recipient", "subject", "text"));
      
   
   you need to give the compiler the -lpthread option, because the used socket
   class is able to work actively (running in its own thread), although that
   feature is not used in this code.
   
   thanks to
      http://www.faqs.org/rfcs/rfc821.html
      http://de.wikipedia.org/wiki/SMTP-Auth     (for authentification)
      http://www.hochfeiler.it/alvise/smtp.htm   (for attachments)
      http://www.ietf.org/rfc/rfc2821.txt        (for BCC)
*/

#ifndef __ALGORITHMAN__SMTP_SOCKET_H
    #define __ALGORITHMAN__SMTP_SOCKET_H

    #include "socket.h"
    #include <vector>
    #include <string>
    #include <stdio.h> // fopen, fread, fclose

    namespace AlgorithMan
    {
    
        class EMail
        {
            public:
                std::string Sender;
                std::vector<std::string> Recipients;
                std::vector<std::string> BCC;
                std::string Subject;
                std::vector<std::string> Text;
                std::vector<std::string> attached_files;

                EMail(std::string Sender, std::string Recipient,               std::string Subject, std::string Text);
                EMail(std::string Sender, std::string Recipient,               std::string Subject, std::vector<std::string> Text);
                EMail(std::string Sender, std::vector<std::string> Recipients, std::string Subject, std::string Text);
                EMail(std::string Sender, std::vector<std::string> Recipients, std::string Subject, std::vector<std::string> Text);
                EMail(std::string Sender, std::string Recipient,               std::string Subject);
                EMail(std::string Sender, std::vector<std::string> Recipients, std::string Subject);
            
                void operator+=(std::string line); // adds a line to the text
                void AddRecipient(std::string Recipient);
                void AddBlindCopyRecipient(std::string BCCRecipient);
                void AddAttachment(std::string path_to_file);
        };


        class SMTP : private Socket
        {
            private:
                std::string server;
                std::string username;
                std::string password;
                
            protected:
                static std::string ToBase64(const unsigned char* str, int length);
                static std::string ToBase64(std::string str);
                static std::string basename(std::string basename);
            
            public:
                SMTP(std::string server, std::string username, std::string password);
                ~SMTP();
                void Send(EMail Mail);
                void Send(std::vector<EMail> Mails);
        };
        
    }

#endif
